package timertests;

/**
 * Each instance of DataSet contains an <CODE>ArrayList</CODE> 
 * of data to be sorted. It provides a <CODE>Traversal</CODE>
 * to generate the data for each of the sorting algorithms that
 * is being tested.
 * 
 * @(#)DataSet  1.0  22 March 2007
 * @author Viera K. Proulx
 */

import java.util.ArrayList;
import stresstests.datasets.*;
import stresstests.interfaces.*;

class DataSet<T>{
  
  /** the actual size of the data set */
  protected int size;       

  /** sequential or random data organization */
  protected String type;          
  
  /** the actual (city) data */
  protected ArrayList<T> data;    

  /**
   * The constructor that reserves the desired space and records
   * how the dataset will be constructed
   * 
   * @param size the designated size of the dataset
   * @param type dataset organization: sequential or random
   */
  protected DataSet(int size, String type){
    // reserve enough space for the expected size
    this.data = new ArrayList<T>(size);
    // but set the actual size to zero
    this.size = 0;    // should grow to the given size
    // how is this data set constructed - include in the results
    this.type = type;
  }
 
  /**
   * The constructor that uses for its data the given 
   * <CODE>ArrayList</CODE> and records
   * how the dataset has been be constructed
   * 
   * @param type dataset organization: sequential or random
   * @param data the <CODE>ArrayList</CODE> that will be this dataset
   */
  protected DataSet(String type, ArrayList<T> data){
    // use the given data
    this.data = data;
    // set the actual size
    this.size = data.size();    
    // how is this data set constructed - include in the results
    this.type = type;
  }
 
  /**
   * Add the given (city) data to this data set
   * 
   * @param city the data to add
   */
  protected void add(T city){
    this.data.add(city);
    this.size = this.size + 1;
  }
  
  /**
   * Provide an iterator for this data set
   * 
   * @return <CODE>Traversal</CODE> for this dataset
   */
  protected Traversal<T> getTraversal(){
    return new TraversalALC<T>(data);
  }
  
  /** Self Test */
  public static void main(String argv[]){
    City northville = 
      new City(48167, "Northville", "MI", 83.479355, 42.426245);
    City pinckney = 
      new City("48169", "Pinckney", "MI", 83.909918, 42.459579);
    City plymouth = 
      new City(48170, "Plymouth", "MI", 83.479946, 42.36882);
    City plymouth2 = new City();
    
    DataSet<City> dset = new DataSet<City>(4, "random");
    
    dset.add(northville);
    dset.add(pinckney);
    dset.add(plymouth);
    dset.add(plymouth2);
    
    System.out.println("Examples of CitiesList-s:");
    Traversal tr = dset.getTraversal();
    
    while (!tr.isEmpty()){
      System.out.println(tr.getFirst().toString());
      tr = tr.getRest(); 
    }
    System.out.println("End of the list.");
  }
}
