package timertests;

/**
 * To represent the result of one run of the timer tests
 * 
 * @(#)Result  22 March 2006
 * @author Viera K. Proulx
 */
public class Result{
  
  /** the size of the dataset that was sorted */
  protected int size;
  
  /** the organization of the dataset: sequential or random */
  protected String type;
  
  /** the name of the algorithm that generated the timing data */
  protected String algoName;
  
  /** the name of the <CODE>Comparator</CODE> that was used to sort data */
  protected String compName;
  
  /** the measured runtime (of the given algorithm on the given dataset
   * of the given size with the given comparator) */
  protected long runtime;
  
   /**
    * The full constructor
    * @param size the size of the dataset that was sorted
    * @param type the organization of the dataset: sequential or random
    * @param algoName the name of the algorithm that generated the timing data
    * @param compName the name of the <CODE>Comparator</CODE> that was used to sort data
    * @param runtime the measured runtime (of the given algorithm on the given dataset
   * of the given size with the given comparator)
    */
  protected Result(int size,
                String type,
                String algoName,
                String compName,
                long runtime){
    this.size = size;
    this.type = type;
    this.algoName = algoName;
    this.compName = compName;
    this.runtime = runtime;
  }
  
  /**
   * Represent the result as a well formatted string
   */ 
  public String toString(){
    // YOU CAN CHANGE THE DELIMITERS USED TO SEPARATE THE PARTS OF THE RESULT
    // TO MAKE IT EASIER TO USE THE RESULTS IN YOUR ANALYSIS
    return "(^" + algoName + "^ ^" + compName + "^ " + size + " ^" + 
           type + "^ " + runtime + ")";
  }
  
  /** Self Test */
  public static void main(String argv[]){
    Result r1 = new Result(8000,
                          "sequential",
                          "By Zip",
                          "Quick Sort with recursively built list",
                          1345);

    Result r2 = new Result(16000,
        "random",
        "Insertion Sort with recursively built list",
        "By Name",
        1345);
    System.out.println(r1.toString());
    System.out.println(r2.toString());
  }
}