package timertests;

import java.util.ArrayList;
import java.util.NoSuchElementException;

import stresstests.datasets.*;
import stresstests.interfaces.*;

/**
 * This class contains the test data for all sorting tests
 * For each data set size: 1000, 2000, 4000, 8000, 16000
 * there is a sequential and random set
 * For size 29470 there is just one complete set
 * 
 * sequential contains a contiguous subset of the source city data
 * random selects data at random from the source city data
 * 
 * @(#)TestData  1.0  22 March 2006
 * @author Viera K. Proulx
 */
public class TestData{
  
  /** original data set of all cities available */
  protected ArrayList<City> alldata = new ArrayList<City>();
  
  /** 15 data sets for tests */
  protected ArrayList<DataSet<City>> datasets = new ArrayList<DataSet<City>>(12);
  
  /**
   * The constructor -- reading the data from the input file
   */
  protected TestData(){
    for(Traversal<City>tr = new TraversalInFile();
        !tr.isEmpty();
        tr = tr.getRest()){
      alldata.add(tr.getFirst());
    }
    System.out.println("read initial data: " + alldata.size());
    buildDataSets();
  }
  
  /**
   * Effect: Construct an <CODE>ArrayList datasets</CODE> of 
   * 15 different sets of data to be used in sorting tests
   */
  protected  void buildDataSets(){
    datasets.add(new DataSet<City>("sequential", 
        this.buildSequentialData(1000)));
    datasets.add(new DataSet<City>("random", 
        this.buildRandomData(1000)));
    datasets.add(new DataSet<City>("sequential", 
        this.buildSequentialData(2000)));
    datasets.add(new DataSet<City>("random", 
        this.buildRandomData(2000)));
    datasets.add(new DataSet<City>("sequential", 
        this.buildSequentialData(4000)));
    datasets.add(new DataSet<City>("random", 
        this.buildRandomData(4000)));
    datasets.add(new DataSet<City>("random", 
        this.buildSequentialData(8000)));
    datasets.add(new DataSet<City>("sequential", 
        this.buildRandomData(8000)));
    datasets.add(new DataSet<City>("random", 
        this.buildSequentialData(16000)));
    datasets.add(new DataSet<City>("sequential", 
        this.buildRandomData(16000)));
    datasets.add(new DataSet<City>("random", 
        this.buildSequentialData(29460)));
    datasets.add(new DataSet<City>("sequential", 
        this.buildSequentialData(29460)));
  }
  
  /**
   * Create the base data set for all tests - reading the data from a file
   */ 
  protected void readData(){
    Traversal<City> tr = new TraversalInFile();
    this.alldata = new ArrayList<City>(30000);
    for(;
    !tr.isEmpty();
    tr.getRest()){
      alldata.add(tr.getFirst());       
    }
    System.out.println("readData produced " + alldata.size() + " elements.");
  }
  
  /**
   * Produce a random dataset of the given size from original base data
   */
  protected ArrayList<City> buildRandomData(int size){
    // make a copy of the original arraylist
    ArrayList<City> temp = (ArrayList<City>)alldata.clone();
    ArrayList<City> oneDataSet = new ArrayList<City>();
    
    if (size < temp.size()){
      
      // selecting elements at random index to add to the data set
      int index = 0;
      int loc;
      
      while (index < size){
        loc = Utilities.randomInt(0, temp.size() - 1);
        oneDataSet.add(temp.get(loc));
        temp.remove(loc);
        index = index + 1;
      }
    }
    System.out.println("Random data set of size " + 
        oneDataSet.size() + " initialized.");
    return oneDataSet;
  }
  
  /**
   * Produce a sequential dataset of the given size from original base data
   */
  protected ArrayList<City> buildSequentialData(int size){
    
    // reserve space for the new  arraylist
    ArrayList<City> oneDataSet = new ArrayList<City>(size);
    
    if (size < this.alldata.size()){
      
      // select at random the starting index for the data set
      int index = 0;
      int loc = Utilities.randomInt(0, this.alldata.size() - size);
      
      while (index < size){
        oneDataSet.add(this.alldata.get(index + loc));
        index = index + 1;
      }
      System.out.println("Sequential data set of size " + 
          oneDataSet.size() + " initialized.");
      return oneDataSet;
    }
    else
      throw new NoSuchElementException(
          "data set of size " + size + "cannot be initialized");
  }
  
  
  /** Self Test */
  public static void main(String[] argv){
    TestData testdata = new TestData();
    System.out.println("alldata size:  " + testdata.alldata.size());
    System.out.println("datasets size: " + testdata.datasets.size());
  }
}